@@ -521,6 +521,7 @@ func (d *DeploymentController) configureIstioGateway(log *istiolog.Scope, gw gat
 
 	input := TemplateInput{
 		Gateway:        &gw,
+		GatewayClass:   string(gw.Spec.GatewayClassName),
 		DeploymentName: model.GetOrDefault(gw.Annotations[annotation.GatewayNameOverride.Name], defaultName),
 		ServiceAccount: model.GetOrDefault(gw.Annotations[annotation.GatewayServiceAccount.Name], defaultName),
 		Ports:          extractServicePorts(gw, listenersFromListenerSets),
@@ -954,6 +955,7 @@ func (d *DeploymentController) canManage(gvr schema.GroupVersionResource, name,
 
 type TemplateInput struct {
 	*gateway.Gateway
+	GatewayClass              string
 	DeploymentName            string
 	ServiceAccount            string
 	Ports                     []corev1.ServicePort
@@ -983,10 +985,10 @@ func extractServicePorts(gw gateway.Gateway, listenerSets []gateway.Listener) []
 	portNums := sets.New[int32]()
 	allListeners := append(slices.Clone(gw.Spec.Listeners), listenerSets...)
 	for i, l := range allListeners {
-		if portNums.Contains(int32(l.Port)) {
+		if portNums.Contains(l.Port) {
 			continue
 		}
-		portNums.Insert(int32(l.Port))
+		portNums.Insert(l.Port)
 		name := sanitizeListenerNameForPort(string(l.Name))
 		if name == "" {
 			// Should not happen since name is required, but in case an invalid resource gets in...
@@ -995,7 +997,7 @@ func extractServicePorts(gw gateway.Gateway, listenerSets []gateway.Listener) []
 		appProtocol := strings.ToLower(string(l.Protocol))
 		svcPorts = append(svcPorts, corev1.ServicePort{
 			Name:        name,
-			Port:        int32(l.Port),
+			Port:        l.Port,
 			AppProtocol: &appProtocol,
 		})
 	}
