@@ -22,6 +22,8 @@ import (
 	cluster "github.com/envoyproxy/go-control-plane/envoy/config/cluster/v3"
 	core "github.com/envoyproxy/go-control-plane/envoy/config/core/v3"
 	endpoint "github.com/envoyproxy/go-control-plane/envoy/config/endpoint/v3"
+	dfpcluster "github.com/envoyproxy/go-control-plane/envoy/extensions/clusters/dynamic_forward_proxy/v3"
+	dfpcommon "github.com/envoyproxy/go-control-plane/envoy/extensions/common/dynamic_forward_proxy/v3"
 	overridehost "github.com/envoyproxy/go-control-plane/envoy/extensions/load_balancing_policies/override_host/v3"
 	roundrobin "github.com/envoyproxy/go-control-plane/envoy/extensions/load_balancing_policies/round_robin/v3"
 	cares "github.com/envoyproxy/go-control-plane/envoy/extensions/network/dns_resolver/cares/v3"
@@ -69,6 +71,20 @@ var passthroughHttpProtocolOptions = protoconv.MessageToAny(&http.HttpProtocolOp
 	},
 })
 
+// nolint
+// revive:disable-next-line
+var passthroughHttpProtocolOptionsWithPreserveHeaderCase = protoconv.MessageToAny(&http.HttpProtocolOptions{
+	CommonHttpProtocolOptions: &core.HttpProtocolOptions{
+		IdleTimeout: durationpb.New(5 * time.Minute),
+	},
+	UpstreamProtocolOptions: &http.HttpProtocolOptions_UseDownstreamProtocolConfig{
+		UseDownstreamProtocolConfig: &http.HttpProtocolOptions_UseDownstreamHttpConfig{
+			HttpProtocolOptions:  preserveCaseFormatterConfig,
+			Http2ProtocolOptions: http2ProtocolOptions(),
+		},
+	},
+})
+
 var preserveCaseFormatterConfig = &core.Http1ProtocolOptions{
 	HeaderKeyFormat: &core.Http1ProtocolOptions_HeaderKeyFormat{
 		HeaderFormat: &core.Http1ProtocolOptions_HeaderKeyFormat_StatefulFormatter{
@@ -87,6 +103,8 @@ type clusterWrapper struct {
 	cluster *cluster.Cluster
 	// httpProtocolOptions stores the HttpProtocolOptions which will be marshaled when build is called.
 	httpProtocolOptions *http.HttpProtocolOptions
+	// isDFPCluster indicates whether the cluster is a dynamic forward proxy cluster
+	isDFPCluster bool
 }
 
 // metadataCerts hosts client certificate related metadata specified in proxy metadata.
@@ -179,6 +197,14 @@ func newClusterWrapper(cluster *cluster.Cluster) *clusterWrapper {
 	}
 }
 
+// newClusterWrapper initializes clusterWrapper with the cluster passed.
+func newDFPClusterWrapper(cluster *cluster.Cluster) *clusterWrapper {
+	return &clusterWrapper{
+		cluster:      cluster,
+		isDFPCluster: true,
+	}
+}
+
 func (cb *ClusterBuilder) applyOverrideHostPolicy(cw *clusterWrapper) {
 	// `locality_weighted_lb_config` is not compatible with
 	// `load_balancing_policy`.
@@ -470,6 +496,30 @@ func (cb *ClusterBuilder) buildCluster(name string, discoveryType cluster.Cluste
 	return ec
 }
 
+// Builds a dynamic forward proxy cluster with DNS cache config, stats configuration
+// and upstream protocol settings.
+func (cb *ClusterBuilder) buildDFPCluster(name string, service *model.Service, port *model.Port) *clusterWrapper {
+	c := &cluster.Cluster{
+		Name:     name,
+		LbPolicy: cluster.Cluster_CLUSTER_PROVIDED,
+		ClusterDiscoveryType: &cluster.Cluster_ClusterType{ClusterType: &cluster.Cluster_CustomClusterType{
+			Name: "envoy.clusters.dynamic_forward_proxy",
+			TypedConfig: protoconv.MessageToAny(&dfpcluster.ClusterConfig{
+				ClusterImplementationSpecifier: &dfpcluster.ClusterConfig_DnsCacheConfig{
+					DnsCacheConfig: &dfpcommon.DnsCacheConfig{
+						Name:            model.BuildDNSCacheName(service.Hostname),
+						DnsLookupFamily: cluster.Cluster_V4_ONLY,
+					},
+				},
+			}),
+		}},
+	}
+	c.AltStatName = util.DelimitedStatsPrefix(name)
+	ec := newDFPClusterWrapper(c)
+	cb.setUpstreamProtocol(ec, port)
+	return ec
+}
+
 // buildInboundCluster constructs a single inbound cluster. The cluster will be bound to
 // `inbound|clusterPort||`, and send traffic to <bind>:<instance.Endpoint.EndpointPort>. A workload
 // will have a single inbound cluster per port. In general this works properly, with the exception of
@@ -630,13 +680,17 @@ func (cb *ClusterBuilder) buildBlackHoleCluster() *cluster.Cluster {
 // generates a cluster that sends traffic to the original destination.
 // This cluster is used to catch all traffic to unknown listener ports
 func (cb *ClusterBuilder) buildDefaultPassthroughCluster() *cluster.Cluster {
+	httpProtocolOptions := passthroughHttpProtocolOptions
+	if shouldPreserveHeaderCase(cb.proxyMetadata, cb.req.Push) {
+		httpProtocolOptions = passthroughHttpProtocolOptionsWithPreserveHeaderCase
+	}
 	cluster := &cluster.Cluster{
 		Name:                 util.PassthroughCluster,
 		ClusterDiscoveryType: &cluster.Cluster_Type{Type: cluster.Cluster_ORIGINAL_DST},
 		ConnectTimeout:       cb.req.Push.Mesh.ConnectTimeout,
 		LbPolicy:             cluster.Cluster_CLUSTER_PROVIDED,
 		TypedExtensionProtocolOptions: map[string]*anypb.Any{
-			v3.HttpProtocolOptionsType: passthroughHttpProtocolOptions,
+			v3.HttpProtocolOptionsType: httpProtocolOptions,
 		},
 	}
 	cluster.AltStatName = util.DelimitedStatsPrefix(util.PassthroughCluster)
@@ -654,7 +708,7 @@ func setH2Options(mc *clusterWrapper) {
 		mc.httpProtocolOptions = &http.HttpProtocolOptions{}
 	}
 	options := mc.httpProtocolOptions
-	if options.UpstreamHttpProtocolOptions == nil {
+	if options.UpstreamProtocolOptions == nil {
 		options.UpstreamProtocolOptions = &http.HttpProtocolOptions_ExplicitHttpConfig_{
 			ExplicitHttpConfig: &http.HttpProtocolOptions_ExplicitHttpConfig{
 				ProtocolConfig: &http.HttpProtocolOptions_ExplicitHttpConfig_Http2ProtocolOptions{
@@ -690,6 +744,11 @@ func (cb *ClusterBuilder) setUseDownstreamProtocol(mc *clusterWrapper) {
 	}
 }
 
+func shouldPreserveHeaderCase(proxyMetadata *model.NodeMetadata, push *model.PushContext) bool {
+	return proxyMetadata.ProxyConfigOrDefault(push.Mesh.GetDefaultConfig()).
+		GetProxyHeaders().GetPreserveHttp1HeaderCase().GetValue()
+}
+
 func http2ProtocolOptions() *core.Http2ProtocolOptions {
 	return &core.Http2ProtocolOptions{}
 }
@@ -710,10 +769,8 @@ func (cb *ClusterBuilder) setUpstreamProtocol(cluster *clusterWrapper, port *mod
 	// Preserve HTTP/1.x traffic header case
 	isExplicitHTTP := port.Protocol.IsHTTP()
 	isAutoProtocol := port.Protocol.IsUnsupported()
-	effectiveProxyConfig := cb.proxyMetadata.ProxyConfigOrDefault(cb.req.Push.Mesh.GetDefaultConfig())
-	preserveHeaderCase := effectiveProxyConfig.GetProxyHeaders().GetPreserveHttp1HeaderCase().GetValue()
 
-	if (isExplicitHTTP || isAutoProtocol) && preserveHeaderCase {
+	if (isExplicitHTTP || isAutoProtocol) && shouldPreserveHeaderCase(cb.proxyMetadata, cb.req.Push) {
 		// Apply the stateful formatter for HTTP/1.x headers
 		if cluster.httpProtocolOptions == nil {
 			cluster.httpProtocolOptions = &http.HttpProtocolOptions{}
