@@ -56,14 +56,16 @@ func (a *index) ServicesCollection(
 	namespaces krt.Collection[*v1.Namespace],
 	meshConfig krt.Singleton[MeshConfig],
 	opts krt.OptionsBuilder,
+	precompute bool,
 ) krt.Collection[model.ServiceInfo] {
-	ServicesInfo := krt.NewCollection(services, a.serviceServiceBuilder(waypoints, namespaces, meshConfig),
+	ServicesInfo := krt.NewCollection(services, a.serviceServiceBuilder(waypoints, namespaces, meshConfig, precompute),
 		append(
 			opts.WithName("ServicesInfo"),
 			krt.WithMetadata(krt.Metadata{
 				multicluster.ClusterKRTMetadataKey: clusterID,
 			}),
 		)...)
+
 	ServiceEntriesInfo := krt.NewManyCollection(serviceEntries, a.serviceEntryServiceBuilder(waypoints, namespaces),
 		append(
 			opts.WithName("ServiceEntriesInfo"),
@@ -71,9 +73,47 @@ func (a *index) ServicesCollection(
 				multicluster.ClusterKRTMetadataKey: clusterID,
 			}),
 		)...)
+	serviceEntryByHostname := krt.NewIndex(ServiceEntriesInfo, "serviceEntryByHostname", func(se ServiceEntryInfo) []string {
+		return []string{fmt.Sprintf("%s/%s", se.Service.Namespace, se.Service.Hostname)}
+	})
+
+	serviceByHostname := krt.NewIndex(ServicesInfo, "serviceByHostname", func(s model.ServiceInfo) []string {
+		return []string{s.Service.Hostname}
+	})
+
+	DedupedServiceEntriesInfo := krt.NewCollection(
+		serviceEntryByHostname.AsCollection(),
+		func(ctx krt.HandlerContext, se krt.IndexObject[string, ServiceEntryInfo]) *model.ServiceInfo {
+			if len(se.Objects) == 0 {
+				return nil
+			}
+
+			s := krt.FetchOne(ctx, ServicesInfo, krt.FilterIndex(serviceByHostname, se.Objects[0].Service.Hostname))
+			if s != nil {
+				// if we have a hostname conflict with a kubernetes service, we should eliminate all the ServiceEntry ServiceInfos for this hostname
+				return nil
+			}
+
+			var oldest *model.ServiceInfo
+			for _, o := range se.Objects {
+				if oldest == nil || o.CreationTime.Before(oldest.CreationTime) {
+					oldest = &o.ServiceInfo
+				}
+			}
+			return oldest
+		}, append(
+			opts.WithName("DedupedServiceEntriesInfo"),
+			krt.WithMetadata(krt.Metadata{
+				multicluster.ClusterKRTMetadataKey: clusterID,
+			}),
+		)...,
+	)
 	WorkloadServices := krt.JoinCollection(
-		[]krt.Collection[model.ServiceInfo]{ServicesInfo, ServiceEntriesInfo},
-		append(opts.WithName("WorkloadService"), krt.WithMetadata(
+		[]krt.Collection[model.ServiceInfo]{
+			ServicesInfo,
+			DedupedServiceEntriesInfo,
+		},
+		append(opts.WithName("WorkloadServices"), krt.WithMetadata(
 			krt.Metadata{
 				multicluster.ClusterKRTMetadataKey: clusterID,
 			},
@@ -123,8 +163,7 @@ func GlobalMergedWorkloadServicesCollection(
 			}
 			waypoints := *waypointsPtr
 			namespaces := cluster.Namespaces()
-			// We can't have duplicate collections (otherwise FetchOne will panic) so use
-			// sync.Once to ensure we only create the collection once and return that same value
+			// N.B Never precompute the service info for remote clusters; the merge function will do that
 			servicesInfo := krt.NewCollection(services, serviceServiceBuilder(
 				waypoints,
 				namespaces,
@@ -140,7 +179,7 @@ func GlobalMergedWorkloadServicesCollection(
 						return ""
 					}
 					return nw.Network
-				}),
+				}, false),
 				append(
 					opts,
 					krt.WithName(fmt.Sprintf("ambient/ServiceServiceInfos[%s]", cluster.ID)),
@@ -174,6 +213,7 @@ func serviceServiceBuilder(
 	localCluster bool,
 	checkServiceScope bool,
 	networkGetter func(ctx krt.HandlerContext) network.ID,
+	precompute bool,
 ) krt.TransformationSingle[*v1.Service, model.ServiceInfo] {
 	return func(ctx krt.HandlerContext, s *v1.Service) *model.ServiceInfo {
 		serviceScope := model.Local
@@ -219,14 +259,20 @@ func serviceServiceBuilder(
 
 		svc := constructService(ctx, s, waypoint, domainSuffix, networkGetter)
 
-		return precomputeServicePtr(&model.ServiceInfo{
+		svcInfo := &model.ServiceInfo{
 			Service:       svc,
 			PortNames:     portNames,
 			LabelSelector: model.NewSelector(s.Spec.Selector),
 			Source:        MakeSource(s),
 			Waypoint:      waypointStatus,
 			Scope:         serviceScope,
-		})
+			CreationTime:  s.CreationTimestamp.Time,
+		}
+		if precompute {
+			return precomputeServicePtr(svcInfo)
+		}
+
+		return svcInfo
 	}
 }
 
@@ -322,6 +368,7 @@ func (a *index) serviceServiceBuilder(
 	waypoints krt.Collection[Waypoint],
 	namespaces krt.Collection[*v1.Namespace],
 	meshConfig krt.Singleton[MeshConfig],
+	precompute bool,
 ) krt.TransformationSingle[*v1.Service, model.ServiceInfo] {
 	return serviceServiceBuilder(
 		waypoints,
@@ -333,6 +380,7 @@ func (a *index) serviceServiceBuilder(
 		func(ctx krt.HandlerContext) network.ID {
 			return a.Network(ctx)
 		},
+		precompute,
 	)
 }
 
@@ -344,15 +392,32 @@ func MakeSource(o controllers.Object) model.TypedObject {
 	}
 }
 
+// ServiceEntryInfo is a wrapper around ServiceInfo that handles key conflicts
+// on hostname.
+type ServiceEntryInfo struct {
+	model.ServiceInfo
+}
+
+func (s ServiceEntryInfo) ResourceName() string {
+	return s.GetNamespace() + "/" + s.GetName() + "/" + s.Service.GetHostname()
+}
+
+func (s ServiceEntryInfo) Equals(other ServiceEntryInfo) bool {
+	return s.ServiceInfo.Equals(other.ServiceInfo)
+}
+
 func (a *index) serviceEntryServiceBuilder(
 	waypoints krt.Collection[Waypoint],
 	namespaces krt.Collection[*v1.Namespace],
-) krt.TransformationMulti[*networkingclient.ServiceEntry, model.ServiceInfo] {
-	return func(ctx krt.HandlerContext, s *networkingclient.ServiceEntry) []model.ServiceInfo {
+) krt.TransformationMulti[*networkingclient.ServiceEntry, ServiceEntryInfo] {
+	return func(ctx krt.HandlerContext, s *networkingclient.ServiceEntry) []ServiceEntryInfo {
 		waypoint, waypointError := fetchWaypointForService(ctx, waypoints, namespaces, s.ObjectMeta)
-		return serviceEntriesInfo(ctx, s, waypoint, waypointError, func(ctx krt.HandlerContext) network.ID {
+		serviceInfos := serviceEntriesInfo(ctx, s, waypoint, waypointError, func(ctx krt.HandlerContext) network.ID {
 			return a.Network(ctx)
 		})
+		return slices.Map(serviceInfos, func(si model.ServiceInfo) ServiceEntryInfo {
+			return ServiceEntryInfo{ServiceInfo: si}
+		})
 	}
 }
 
@@ -381,6 +446,13 @@ func serviceEntriesInfo(
 	if wperr != nil {
 		waypoint.Error = wperr
 	}
+
+	// Warn if we have a dynamic DNS service entry with no egress waypoint
+	// This configuration will not be shared with the dataplane
+	if s.Spec.Resolution == v1alpha3.ServiceEntry_DYNAMIC_DNS && (w == nil || wperr != nil) {
+		log.Warnf("ServiceEntry %s/%s has dynamic DNS resolution but no valid waypoint", s.Namespace, s.Name)
+	}
+
 	return slices.Map(constructServiceEntries(ctx, s, w, networkGetter), func(e *workloadapi.Service) model.ServiceInfo {
 		return precomputeService(model.ServiceInfo{
 			Service:       e,
@@ -388,6 +460,7 @@ func serviceEntriesInfo(
 			LabelSelector: sel,
 			Source:        MakeSource(s),
 			Waypoint:      waypoint,
+			CreationTime:  s.CreationTimestamp.Time,
 		})
 	})
 }
@@ -424,22 +497,36 @@ func constructServiceEntries(
 
 	var lb *workloadapi.LoadBalancing
 
-	trafficDistribution := model.GetTrafficDistribution(nil, svc.Annotations)
-	switch trafficDistribution {
-	case model.TrafficDistributionPreferSameZone:
-		lb = preferSameZoneLoadBalancer
-	case model.TrafficDistributionPreferSameNode:
-		lb = preferSameNodeLoadBalancer
+	// When resolution is NONE, we want to passthrough traffic to the destination address.
+	// In this case, we can skip attempting to get a traffic distribution because it is not applicable.
+	if svc.Spec.Resolution == v1alpha3.ServiceEntry_NONE {
+		lb = &workloadapi.LoadBalancing{
+			Mode: workloadapi.LoadBalancing_PASSTHROUGH,
+			// HealthPolicy is primarily aesthetic in this case, making the WDS easier to understand.
+			// We pass through to the endpoint called, so it seems weird if WDS has a OnlyHealthy LoadBalancer setting.
+			HealthPolicy: workloadapi.LoadBalancing_ALLOW_ALL,
+		}
+	} else {
+		trafficDistribution := model.GetTrafficDistribution(nil, svc.Annotations)
+		switch trafficDistribution {
+		case model.TrafficDistributionPreferSameZone:
+			lb = preferSameZoneLoadBalancer
+		case model.TrafficDistributionPreferSameNode:
+			lb = preferSameNodeLoadBalancer
+		}
 	}
 
 	// TODO this is only checking one controller - we may be missing service vips for instances in another cluster
 	res := make([]*workloadapi.Service, 0, len(svc.Spec.Hosts))
 	for _, h := range svc.Spec.Hosts {
-		// if we have no user-provided hostsAddresses and h is not wildcarded and we have hostsAddresses supported resolution
+		// if we have no user-provided hostsAddresses and we have hostsAddresses supported resolution
 		// we can try to use autoassigned hostsAddresses
 		hostsAddresses := addresses
-		if len(hostsAddresses) == 0 && !host.Name(h).IsWildCarded() && svc.Spec.Resolution != v1alpha3.ServiceEntry_NONE {
-			if hostsAddrs, ok := autoassignedHostAddresses[h]; ok {
+		if len(hostsAddresses) == 0 && svc.Spec.Resolution != v1alpha3.ServiceEntry_NONE {
+			// block auto IP assignment for wildcards unless the resolution type is DYNAMIC_DNS
+			if host.Name(h).IsWildCarded() && svc.Spec.Resolution != v1alpha3.ServiceEntry_DYNAMIC_DNS {
+				log.Debugf("autoassigned IPs only support for wildcarded hosts with dynamic DNS resolution, skipping %s", h)
+			} else if hostsAddrs, ok := autoassignedHostAddresses[h]; ok {
 				hostsAddresses = slices.Map(hostsAddrs, func(e netip.Addr) *workloadapi.NetworkAddress {
 					return toNetworkAddressFromIP(e, networkGetter(ctx))
 				})
