@@ -31,8 +31,7 @@ func NewTypeMap() TypeMap {
 }
 
 func Set[T any](t TypeMap, v T) {
-	interfaceType := reflect.TypeOf((*T)(nil)).Elem()
-	t.inner[interfaceType] = v
+	t.inner[reflect.TypeFor[T]()] = v
 }
 
 func Get[T any](t TypeMap) *T {
